@@ -57,9 +57,22 @@ char *login_program = LOGIN_PROGRAM;
 int	utmp_len = MAXHOSTNAMELEN;	/* sizeof(init_request.host) */
 #else	/* NEWINIT*/
 
-char	wtmpf[]	= "/usr/adm/wtmp";
-char	utmpf[] = "/etc/utmp";
+#ifdef HAVE_UTMP_H
+#include <utmp.h>
+#endif
 
+#ifdef _PATH_WTMP
+char	wtmpf[] = _PATH_WTMP;
+#else
+char	wtmpf[]	= "/usr/adm/wtmp";
+#endif
+
+#ifdef _PATH_UTMP
+char 	utmpf[] = _PATH_UTMP;
+#else
+char	utmpf[] = "/etc/utmp";
+#endif
+  
 # ifdef CRAY
 #include <tmpdir.h>
 #include <sys/wait.h>
@@ -82,7 +95,7 @@ extern struct sysv sysv;
 #endif	/* NEWINIT */
 
 #ifdef	STREAMSPTY
-#ifdef HAS_SAC
+#ifdef HAVE_SAC_H
 #include <sac.h> 
 #endif
 #include <sys/stropts.h>
@@ -91,7 +104,7 @@ extern struct sysv sysv;
 #define SCPYN(a, b)	(void) strncpy(a, b, sizeof(a))
 #define SCMPN(a, b)	strncmp(a, b, sizeof(a))
 
-#ifdef	STREAMS
+#ifdef	HAVE_SYS_STREAM_H
 #include <sys/stream.h>
 #endif
 #ifdef __hpux
@@ -927,9 +940,12 @@ getptyslave()
 #ifdef	USE_TERMIO
 	ttyfd = t;
 #endif
-	if (ioctl(pty, I_PUSH, "pckt") < 0)
+	if (ioctl(pty, I_PUSH, "pckt") < 0) {
+#ifndef _AIX
 		fatal(net, "I_PUSH pckt");
 #endif
+	}
+#endif
 
 	/*
 	 * set up the tty modes as we like them to be.
@@ -1122,7 +1138,7 @@ startslave(host, autologin, autoname)
 	SCPYN(request.gen_id, gen_id);
 	SCPYN(request.tty_id, &line[8]);
 	SCPYN(request.host, host);
-	SCPYN(request.term_type, terminaltype ? terminaltype : "network");
+	SCPYN(request.term_type, *terminaltype ? terminaltype : "network");
 #if	!defined(UNICOS5)
 	request.signal = SIGCLD;
 	request.pid = getpid();
@@ -1289,7 +1305,7 @@ start_login(host, autologin, name)
 
 		if (pty > 2) {
 			register char *cp;
-			char speed[128];
+			char speed[1024];
 			int isecho, israw, xpty, len;
 			extern int def_rspeed;
 #  ifndef LOGIN_HOST
@@ -1326,7 +1342,13 @@ start_login(host, autologin, name)
 			len = strlen(name)+1;
 			write(xpty, name, len);
 			write(xpty, name, len);
-			sprintf(speed, "%s/%d", (cp = getenv("TERM")) ? cp : "",
+			memset(speed, 0, sizeof(speed));
+			strncpy(speed,
+				(cp = getenv("TERM")) ? cp : "",
+				sizeof(speed)-1-(10*sizeof(def_rspeed)/4)-1);
+			/* 1 for /, () for the number, 1 for trailing 0. */
+			sprintf(speed + strlen(speed),
+				"/%d",
 				(def_rspeed > 0) ? def_rspeed : 9600);
 			len = strlen(speed)+1;
 			write(xpty, speed, len);
