@@ -97,7 +97,7 @@ int	registerd_host_only = 0;
 #ifdef HAVE_SYS_TTY_H
 # include "/usr/include/sys/tty.h"
 #endif
-#ifdef  HAS_PTYVAR
+#ifdef  HAVE_SYS_PTYVAR_H
 # include <sys/ptyvar.h>
 #endif
 
@@ -140,6 +140,10 @@ int debug = 0;
 int keepalive = 1;
 char *progname;
 
+int maxhostlen = 0;
+int always_ip = 0;
+int stripdomain = 1;
+
 extern void usage P((void));
 
 /*
@@ -149,6 +153,7 @@ extern void usage P((void));
  */
 char valid_opts[] = {
 	'd', ':', 'h', 'k', 'L', ':', 'n', 'S', ':', 'U',
+	'w',
 #ifdef	AUTHENTICATION
 	'a', ':', 'X', ':',
 #endif
@@ -179,6 +184,28 @@ char valid_opts[] = {
 	'\0'
 };
 
+#include <sys/utsname.h>
+static char *
+get_default_IM()
+{
+	struct utsname name;
+	static char banner[1024];
+	
+	if (uname(&name) < 0)
+		sprintf(banner, "\r\nError getting hostname: %s\r\n",
+		    strerror(errno));
+        else {
+#if defined(_AIX)
+		sprintf(banner, "\r\n    %%h (%s release %s.%s) (%%t)\r\n\r\n",
+		    name.sysname, name.version, name.release);
+#else
+		sprintf(banner, "\r\n    %%h (%s release %s %s) (%%t)\r\n\r\n",
+		    name.sysname, name.release, name.version);
+#endif
+	}
+	return banner;
+}
+
 main(argc, argv)
 	int argc;
 	char *argv[];
@@ -209,7 +236,7 @@ main(argc, argv)
 	highpty = getnpty();
 #endif /* CRAY */
 
-	while ((ch = getopt(argc, argv, valid_opts)) != EOF) {
+	while ((ch = getopt(argc, argv, valid_opts)) != -1) {
 		switch(ch) {
 
 #ifdef	AUTHENTICATION
@@ -382,7 +409,7 @@ main(argc, argv)
 			break;
 #endif	/* SecurID */
 		case 'S':
-#ifdef	HAS_GETTOS
+#ifdef	HAVE_GETTOSBYNAME
 			if ((tos = parsetos(optarg, "tcp")) < 0)
 				fprintf(stderr, "%s%s%s\n",
 					"telnetd: Bad TOS argument '", optarg,
@@ -416,6 +443,36 @@ main(argc, argv)
 			auth_disable_name(optarg);
 			break;
 #endif	/* AUTHENTICATION */
+		case 'w':
+			if (!strcmp(optarg, "ip"))
+				always_ip = 1;
+			else {
+				char *cp;
+				cp = strchr(optarg, ',');
+				if (cp == NULL)
+					maxhostlen = atoi(optarg);
+				else if (*(++cp)) {
+					if (!strcmp(cp, "striplocal"))
+						stripdomain = 1;
+					else if (!strcmp(cp, "nostriplocal"))
+						stripdomain = 0;
+					else {
+						usage();
+					}
+					*(--cp) = '\0';
+					maxhostlen = atoi(optarg);
+				}
+			}
+			break;
+		case 'u':
+			maxhostlen = atoi(optarg);
+			break;
+		case 'i':
+			always_ip = 1;
+			break;
+		case 'N':
+			stripdomain = 0;
+			break;
 
 		default:
 			fprintf(stderr, "telnetd: %c: unknown option\n", ch);
@@ -563,7 +620,7 @@ main(argc, argv)
 
 #if	defined(IPPROTO_IP) && defined(IP_TOS)
 	{
-# if	defined(HAS_GETTOS)
+# if	defined(HAVE_GETTOSBYNAME)
 		struct tosent *tp;
 		if (tos < 0 && (tp = gettosbyname("telnet", "tcp")))
 			tos = tp->t_tos;
@@ -619,13 +676,14 @@ usage()
 #ifdef	SecurID
 	fprintf(stderr, " [-s]");
 #endif
-#ifdef	HAS_GETTOS
+#ifdef	HAVE_GETTOSBYNAME
 	fprintf(stderr, " [-S tos]");
 #endif
 #ifdef	AUTHENTICATION
 	fprintf(stderr, " [-X auth-type]");
 #endif
-	fprintf(stderr, " [-u utmp_hostname_length] [-U]");
+	fprintf(stderr, " [-u utmp_hostname_length] [-U]\n");
+	fprintf(stderr, " [-w [ip|maxhostlen[,[no]striplocal]]]\n");
 	fprintf(stderr, " [port]\n");
 	exit(1);
 }
@@ -865,6 +923,7 @@ terminaltypeok(s)
 char *hostname;
 char host_name[MAXDNAME];
 char remote_host_name[MAXDNAME];
+char *rhost_sane;
 
 #ifndef	convex
 extern void telnet P((int, int));
@@ -910,6 +969,12 @@ pty_init();
 	}
 #endif	/* _SC_CRAY_SECURE_SYS */
 
+	retval = pty_make_sane_hostname(who, maxhostlen,
+					stripdomain, always_ip,
+					&rhost_sane);
+	if (retval) {
+		fatal(net, error_message(retval));
+	}
 	/* get name of connected client */
 	hp = gethostbyaddr((char *)&who->sin_addr, sizeof (struct in_addr),
 		who->sin_family);
@@ -917,24 +982,13 @@ pty_init();
 	if (hp == NULL && registerd_host_only) {
 		fatal(net, "Couldn't resolve your address into a host name.\r\n\
          Please contact your net administrator");
-	} else if (hp ) {
-		host = hp->h_name;
-	} else {
-		host = inet_ntoa(who->sin_addr);
 	}
-	/*
-	 * We must make a copy because Kerberos is probably going
-	 * to also do a gethost* and overwrite the static data...
-	 */
-	strncpy(remote_host_name, host, sizeof(remote_host_name)-1);
-	remote_host_name[sizeof(remote_host_name)-1] = 0;
-	host = remote_host_name;
 
 	(void) gethostname(host_name, sizeof (host_name));
 	hostname = host_name;
 
 #if	defined(AUTHENTICATION) || defined(ENCRYPTION)
-	auth_encrypt_init(hostname, host, "TELNETD", 1);
+	auth_encrypt_init(hostname, rhost_sane, "TELNETD", 1);
 #endif
 
 	init_env();
@@ -958,7 +1012,7 @@ pty_init();
 	 * Start up the login process on the slave side of the terminal
 	 */
 #ifndef	convex
-	startslave(host, level, user_name);
+	startslave(rhost_sane, level, user_name);
 
 #if	defined(_SC_CRAY_SECURE_SYS)
 	if (secflag) {
@@ -971,7 +1025,7 @@ pty_init();
 
 	telnet(net, pty);  /* begin server processing */
 #else
-	telnet(net, pty, host);
+	telnet(net, pty, rhost_sane);
 #endif
 	/*NOTREACHED*/
 }  /* end of doit */
@@ -1210,7 +1264,7 @@ telnet(f, p, host)
 		if (IM == 0)
 			IM = "";
 	} else {
-		IM = DEFAULT_IM;
+		IM = get_default_IM();
 		HEstr = 0;
 	}
 	edithost(HEstr, host_name);
@@ -1479,6 +1533,7 @@ telnet(f, p, host)
 		if (FD_ISSET(p, &obits) && (pfrontp - pbackp) > 0)
 			ptyflush();
 	}
+	(void) signal(SIGCHLD, SIG_DFL);
 	cleanup(0);
 }  /* end of telnet */
 	
