@@ -10,12 +10,12 @@
 #include "mit-copyright.h"
 
 #include "krb.h"
+#include "k5-int.h"
 
 #include <stdio.h>
 #include <string.h>
 #include <errno.h>
 #include <sys/stat.h>
-#include <sys/file.h>
 #include <fcntl.h>
 
 #ifdef TKT_SHMEM
@@ -31,6 +31,8 @@
 extern int errno;
 extern int krb_debug;
 
+void tf_close();
+
 #ifdef TKT_SHMEM
 char *krb_shm_addr;
 static char *tmp_shm_addr;
@@ -59,42 +61,6 @@ int utimes(path, times)
   return utime(path,&tv);
 }
 #endif
-#ifndef LOCK_SH
-#define   LOCK_SH   1    /* shared lock */
-#define   LOCK_EX   2    /* exclusive lock */
-#define   LOCK_NB   4    /* don't block when locking */
-#define   LOCK_UN   8    /* unlock */
-#endif
-
-
-#ifdef POSIX_FILE_LOCKS
-
-/*
- * This function emulates a subset of flock()
- */
-int emul_flock(fd, cmd)
-    int fd, cmd;
-{
-    static struct flock flock_zero;
-    struct flock f;
-
-    f = flock_zero;
-
-    memset(&f, 0, sizeof (f));
-
-    if (cmd & LOCK_UN)
-	f.l_type = F_UNLCK;
-    if (cmd & LOCK_SH)
-	f.l_type = F_RDLCK;
-    if (cmd & LOCK_EX)
-	f.l_type = F_WRLCK;
-
-    return fcntl(fd, (cmd & LOCK_NB) ? F_SETLK : F_SETLKW, &f);
-}
-
-#define flock(f,c)	emul_flock(f,c)
-#endif
-
 
 /*
  * fd must be initialized to something that won't ever occur as a real
@@ -183,14 +149,21 @@ int tf_init(tf_name, rw)
     int rw;
 {
     int     wflag;
-    uid_t   me, getuid();
-    struct stat stat_buf;
+    uid_t   me= getuid();
+    struct stat stat_buf, stat_buffd;
 #ifdef TKT_SHMEM
     char shmidname[MAXPATHLEN]; 
     FILE *sfp;
     int shmid;
 #endif
 
+    if (!krb5__krb4_context) {
+	    if (krb5_init_context(&krb5__krb4_context))
+		    return TKT_FIL_LCK;
+    }
+
+    me = getuid();
+
     switch (rw) {
     case R_TKT_FIL:
 	wflag = 0;
@@ -207,25 +180,9 @@ int tf_init(tf_name, rw)
     if (tf_name == 0)
 	tf_name = tkt_string();
 
-    if (lstat(tf_name, &stat_buf) < 0)
-	switch (errno) {
-	case ENOENT:
-	    return NO_TKT_FIL;
-	default:
-	    return TKT_FIL_ACC;
-	}
-    me = getuid();
-    if ((stat_buf.st_uid != me && me != 0) ||
-	((stat_buf.st_mode & S_IFMT) != S_IFREG))
-	return TKT_FIL_ACC;
 #ifdef TKT_SHMEM
     (void) strcpy(shmidname, tf_name);
     (void) strcat(shmidname, ".shm");
-    if (stat(shmidname,&stat_buf) < 0)
-	return(TKT_FIL_ACC);
-    if ((stat_buf.st_uid != me && me != 0) ||
-	((stat_buf.st_mode & S_IFMT) != S_IFREG))
-	return TKT_FIL_ACC;
 #endif /* TKT_SHMEM */
 
     /*
@@ -240,15 +197,51 @@ int tf_init(tf_name, rw)
 #ifdef TKT_SHMEM
     sfp = fopen(shmidname, "r");	/* only need read/write on the
 					   actual tickets */
-    if (sfp == 0)
-    	return TKT_FIL_ACC;
+    if (sfp == 0) {
+        switch(errno) {
+        case ENOENT:
+	    return NO_TKT_FIL;
+	default:
+	    return TKT_FIL_ACC;
+	}
+    }
+
+    /* lstat() and fstat() the file to check that the file we opened is the *
+     * one we think it is, and to check ownership.                          */
+    if ((fstat(sfp->_file, &stat_buffd) < 0) || 
+	(lstat(shmidname, &stat_buf) < 0)) {
+        (void) close(fd);
+	fd = -1;
+	switch(errno) {
+	case ENOENT:
+	    return NO_TKT_FIL;
+	default:
+	    return TKT_FIL_ACC;
+	}
+    }
+    /* Check that it's the right file */
+    if ((stat_buf.st_ino != stat_buffd.st_ino) ||
+	(stat_buf.st_dev != stat_buffd.st_dev)) {
+        (void) close(fd);
+	fd = -1;
+	return TKT_FIL_ACC;
+    }
+    /* Check ownership */
+    if ((stat_buffd.st_uid != me && me != 0) ||
+	((stat_buffd.st_mode & S_IFMT) != S_IFREG)) {
+        (void) close(fd);
+	fd = -1;
+	return TKT_FIL_ACC;
+    }
+
+
+
     shmid = -1;
     {
 	char buf[BUFSIZ];
 	int val;			/* useful for debugging fscanf */
 	/* We provide our own buffer here since some STDIO libraries
 	   barf on unbuffered input with fscanf() */
-
 	setbuf(sfp, buf);
 	if ((val = fscanf(sfp,"%d",&shmid)) != 1) {
 	    (void) fclose(sfp);
@@ -280,11 +273,47 @@ int tf_init(tf_name, rw)
     if (wflag) {
 	fd = open(tf_name, O_RDWR, 0600);
 	if (fd < 0) {
+	    switch(errno) {
+	    case ENOENT:
+	        return NO_TKT_FIL;
+	    default:
+	        return TKT_FIL_ACC;
+	  }
+	}
+	/* lstat() and fstat() the file to check that the file we opened is the *
+	 * one we think it is, and to check ownership.                          */
+	if ((fstat(fd, &stat_buffd) < 0) || 
+	    (lstat(tf_name, &stat_buf) < 0)) {
+	    (void) close(fd);
+	    fd = -1;
+	    switch(errno) {
+	    case ENOENT:
+	        return NO_TKT_FIL;
+	    default:
+	        return TKT_FIL_ACC;
+	    }
+	}
+	/* Check that it's the right file */
+	if ((stat_buf.st_ino != stat_buffd.st_ino) ||
+	    (stat_buf.st_dev != stat_buffd.st_dev)) {
+	    (void) close(fd);
+	    fd = -1;
 	    return TKT_FIL_ACC;
 	}
-	if (flock(fd, LOCK_EX | LOCK_NB) < 0) {
+	/* Check ownership */
+	if ((stat_buffd.st_uid != me && me != 0) ||
+	    ((stat_buffd.st_mode & S_IFMT) != S_IFREG)) {
+	    (void) close(fd);
+	    fd = -1;
+	    return TKT_FIL_ACC;
+	}
+	if (krb5_lock_file(krb5__krb4_context, fd,
+			   KRB5_LOCKMODE_EXCLUSIVE |
+			   KRB5_LOCKMODE_DONTBLOCK) < 0) {
 	    sleep(TF_LCK_RETRY);
-	    if (flock(fd, LOCK_EX | LOCK_NB) < 0) {
+	    if (krb5_lock_file(krb5__krb4_context, fd,
+			   KRB5_LOCKMODE_EXCLUSIVE |
+			   KRB5_LOCKMODE_DONTBLOCK) < 0) {
 		(void) close(fd);
 		fd = -1;
 		return TKT_FIL_LCK;
@@ -299,11 +328,47 @@ int tf_init(tf_name, rw)
 
     fd = open(tf_name, O_RDONLY, 0600);
     if (fd < 0) {
+        switch(errno) {
+	case ENOENT:
+	    return NO_TKT_FIL;
+	default:
+	    return TKT_FIL_ACC;
+	}
+    }
+    /* lstat() and fstat() the file to check that the file we opened is the *
+     * one we think it is, and to check ownership.                          */
+    if ((fstat(fd, &stat_buffd) < 0) || 
+	(lstat(tf_name, &stat_buf) < 0)) {
+        (void) close(fd);
+	fd = -1;
+	switch(errno) {
+	case ENOENT:
+	    return NO_TKT_FIL;
+	default:
+	    return TKT_FIL_ACC;
+	}
+    }
+    /* Check that it's the right file */
+    if ((stat_buf.st_ino != stat_buffd.st_ino) ||
+	(stat_buf.st_dev != stat_buffd.st_dev)) {
+        (void) close(fd);
+	fd = -1;
 	return TKT_FIL_ACC;
     }
-    if (flock(fd, LOCK_SH | LOCK_NB) < 0) {
+    /* Check ownership */
+    if ((stat_buffd.st_uid != me && me != 0) ||
+	((stat_buffd.st_mode & S_IFMT) != S_IFREG)) {
+        (void) close(fd);
+	fd = -1;
+	return TKT_FIL_ACC;
+    }
+    if (krb5_lock_file(krb5__krb4_context, fd,
+			   KRB5_LOCKMODE_SHARED |
+			   KRB5_LOCKMODE_DONTBLOCK) < 0) {
 	sleep(TF_LCK_RETRY);
-	if (flock(fd, LOCK_SH | LOCK_NB) < 0) {
+	if (krb5_lock_file(krb5__krb4_context, fd,
+			   KRB5_LOCKMODE_SHARED |
+			   KRB5_LOCKMODE_DONTBLOCK) < 0) {
 	    (void) close(fd);
 	    fd = -1;
 	    return TKT_FIL_LCK;
@@ -435,7 +500,7 @@ int tf_get_cred(c)
  * The return value is not defined.
  */
 
-tf_close()
+void tf_close()
 {
     if (!(fd < 0)) {
 #ifdef TKT_SHMEM
@@ -447,7 +512,9 @@ tf_close()
 	    krb_shm_addr = 0;
 	}
 #endif /* TKT_SHMEM */
-	(void) flock(fd, LOCK_UN);
+	if (!krb5__krb4_context)
+		krb5_init_context(&krb5__krb4_context);
+	(void) krb5_lock_file(krb5__krb4_context, fd, KRB5_LOCKMODE_UNLOCK);
 	(void) close(fd);
 	fd = -1;		/* see declaration of fd above */
     }
